; X Y Z -Positionsanzeige fr die CNC
; zustzlich Steuerung eines Laserkopfes

.equ __33EP64GP504, 1 
.include "p33EP64GP504.inc"
.include "ram.inc"
.include "allgdef.inc"
.global __DefaultInterrupt
.global __OscillatorFail
.global __HardTrapError
.global __StackError,__MathError,__DMACError,__SoftTrapError

; Debuger
config	__FICD,ICS_PGD1 & JTAGEN_OFF
; OSC2 auf normal IO
config  __FOSC,OSCIOFNC_ON  & FCKSM_CSECME
; Oszillator auf intern 7,37MHz, und Taktumschaltung erlauben
config 	__FOSCSEL,FNOSC_FRC; & IESO_ON
; Wachhund = aus
config	__FWDT,FWDTEN_OFF

.global	__reset
.global __T1Interrupt
.global __INT1Interrupt
.global __INT2Interrupt

; RAM
.section	MAINRAM, bss, address(0x1050)

.text	

__reset:
 
;stack setzen
 	mov 	#stackanfang,w15
	mov 	#stackanfang+100,W0
	mov 	W0,_SPLIM
; RAM lschen
 	mov		#0x1050,w0
 	repeat	#(ramende-ramanfang)
 	clr		[w0++]

; Oszillator schalten auf 30 MIPS
	call	OszillatorSetzen
	 
; Port A
	mov		#0b1111111111101111,w0 ; Eingnge b10...b1,b4=OC2,b0 = Analog
	mov		w0,TRISA	
	mov		#0b0000011111101110,w0
	mov		w0,CNPUA      			; Pull-Up (=1)ACCBH
	mov		#0b0000000000000001,w0
	mov		w0,ANSELA	
; Port B
	mov		#0b0000000000000001,w0	; 0 = Ausgnge 1 = Eingnge
	mov		w0,TRISB
	clr		w0
	mov		w0,ANSELB
	mov		w0,CNPUA
	mov		w0,ODCB 				; OpenDrain (=1)
; Port C
	mov		#0b0000001111111111,w0 	; 1 = Eingang, 0 = Ausgang
	mov		w0,TRISC 
	mov		#0b0000000000000100,w0
	mov		w0,CNPUC      			; Pull-Up (=1)
	clr		w0
	mov		w0,ODCC               	; OpenDrain
	mov		#0b0000000000000011,w0
	mov		w0,ANSELC	     		; Analog
	
; Timer 1 Interrupt alle 10us
	clr		T1CON
	clr		TMR1
	mov		#0b000000000,w0 	; int Clk 32ns
	mov		w0,T1CON
	mov		#312,w0				; 156 x 32ns = ca. 10us
	mov		w0,PR1
	BCLR 	IFS0,#T1IF 
	bset 	IEC0,#T1IE
	bset	T1CON,#TON
	
; PWM OC2 mit Timer3: PWM fr Laserkopf
	CLR 	OC2CON1
	CLR 	OC2CON2
	mov		#0b0000010000000101,w0
	mov		w0,OC2CON1
	mov		#0b0000000000001101,w0  ; Trigger = Timer3
	mov		w0,OC2CON2
	mov		#0b0000000000010000,w0	; Timer3-Takt = 34 x 8 = 272ns
	mov		w0,T3CON
	clr		w0
	mov		w0,OC2R
	MOV 	#0x001, w0 		; erstmal fast aus
	MOV 	w0, OC2RS       ; Tastverhltnis hier einstellbar
	MOV 	#256, w0 		; Periode 70us (ca. 14,4kHz)
	MOV 	w0, PR3
	bset	T3CON, #TON
	
; Timer 4 fr X-Achse externen Interrupt 1
	mov		#0b0000000000010000,w0		; 1:8 = 256ns-Takt
	mov		w0,T4CON
	clr		TMR4
	mov		#0xffff,w0
	mov		w0,PR4					; Soll bis zum Ende Timern
	bset	T4CON,#TON

; Timer 5 fr Y-Achse externen Interrupt 2
	mov		#0b0000000000010000,w0		; 1:8 = 256ns-Takt
	mov		w0,T5CON
	clr		TMR5
	mov		#0xfffe,w0
	mov		w0,PR5					; Soll bis zum Ende Timern
	bset	T5CON,#TON	

; OC2 auf Pin43 (RP20)
	mov		#0b010001,w0
	mov		w0,RPOR0
; Int1 auf pin 37 (RPI52)
	mov		#0b0011010000000000,w0
	mov		w0,RPINR0
; Int2 auf pin 38 (RPI53)
	mov		#0b0110101,w0
	mov		w0,RPINR1

; PSV aktivieren
	movpag	#psvpage(psvanfang),DSRPAG
	
; Display, ADW, Drehrichtung
	call	Lcdinit
	call	ADWinit
	btss	PORTA,#kodier_a
	bset	merker,#andersrum
	
; Ext. Interrupt
	bset	IEC1,#INT1IE	
	bset	IEC1,#INT2IE	

; Hauptschleife	-------------------------------------
hp:
	nop
	call	Input
	nop
	call	Betrieb
 	nop
	call	Monitor
	nop
	call	Lcdbed
	nop
	call	ADWcon
	nop
	call	Lasercon
	nop
	call	Tim45con
	nop
	goto	hp

; Hauptschleife Ende ---------------------------------

; Interrupt wird alle 10us aktiv, Z-Achse zhlen
; reservierte Speicher / Register:
; (D)quittung = Merker fr bereits erkannte Takte
; W13 = Vorzhler, W10 = als Arb-Register
__T1Interrupt:
	BCLR 	IFS0,#T1IF ; Reset Timer1 interrupt flag
	mov		PORTC,w10
	btss	W10,#sm_iz
	bra		t1int2		; H?
	btsc	quittung,#sm_iz
	retfie
	bset	quittung,#sm_iz
	btsc	merker,#andersrum
	btg		W10,#sm_rz
	btss	W10,#sm_rz
	bra		t1int1
	bset	merker,#zimpulsminus
	dec		w13,w13
	retfie
t1int1:
	bset	merker,#zimpulsplus
	inc		w13,w13
	retfie
t1int2:
	bclr	quittung,#sm_iz
	retfie

; --------------------------------------

; INT1
; X-Achse, Register nur hierfr: W11
; max.512ns
__INT1Interrupt:
	push	w0
	bclr	IFS1,#INT1IF
	btss	PORTC,#sm_ix	; Strimpuls?
	bra		int1c			; ja!
	bset	merker,#ximpuls
	mov		TMR4,w0
	clr		TMR4
	mov		w0,xweg
	mov		PORTC,w0
	btsc	merker,#andersrum
	btg		W0,#sm_rx
	btss	W0,#sm_rx
	bra		int1a
	dec		w11,w11
	pop		w0
	retfie
int1a:
	inc		w11,w11
int1b:
	pop		w0
	retfie
int1c:
	inc		testram+8
	bra		int1b
	
; INT2
; Y-Achse, Register nur hierfr: W12, ACCAL

__INT2Interrupt:
	push	w0
	bclr	IFS1,#INT2IF
	btss	PORTC,#sm_iy	; Strimpuls?
	bra		int2c			; ja!
	bset	merker,#yimpuls
	mov		TMR5,w0
	clr		TMR5
	mov		w0,yweg
	mov		PORTC,w0
	btsc	merker,#andersrum
	btg		w0,#sm_ry
	btss	w0,#sm_ry
	bra		int2a
	dec		w12,w12
	pop		w0
	retfie
int2a:
	inc		w12,w12
int2b:
	pop		w0
	retfie
int2c:
	inc		testram+10
	bra		int2b

; --------------------------------------

__DefaultInterrupt:
	nop
	nop
	retfie

__OscillatorFail:
	nop
	nop
	retfie

__HardTrapError:
	nop
	nop
	retfie

__StackError:
	nop
	nop
	retfie

__MathError:
	nop
	nop
	retfie

__DMACError:
	nop
	nop
	retfie

__SoftTrapError:
    nop  
    nop
	RETFIE 
					